#[allow(clippy::module_inception)]
pub mod bot {

    use crate::{config_reader::config_reader::BotUserInfo, utils};
    use log::{info, debug};
    use matrix_sdk::{
        Client,
        config::{SyncSettings, RequestConfig},
        event_handler::Ctx,
        room::{Joined, Room, RoomMember},
        ruma::{
            UserId, ServerName, user_id, RoomId,
            events::room::message::{OriginalSyncRoomMessageEvent,
                                    RoomMessageEventContent},
        },
    };
    use crate::utils::utils::*;

    #[allow(dead_code)]
    #[derive(Clone)]
    pub struct Bot {
        client: Client,
        joined_room: Joined,
        swear_list: Vec<String>
    }

    impl Bot {
        pub async fn bot_login(creds: BotUserInfo) {
            let user_id = creds.user_id.as_str();
            let password = creds.password.as_str();
            // let homeserver = creds.homeserver.as_str();
            let room = RoomId::parse(creds.room.as_str()).unwrap();
            // let client = Client::builder().homeserver_url(homeserver).build().await.unwrap();

            let user = UserId::parse(user_id).expect("Unable to parse bot user id");
            let server_name = ServerName::parse(user.server_name()).unwrap();
            let request_config = RequestConfig::new().force_auth();
            let client = Client::builder()
                .server_name(&server_name)
                .request_config(request_config)
                .build()
                .await.
                unwrap();

            info!("Logging in…");
            let response = client
                .login_username(user.localpart(), password)
                .initial_device_display_name("Matrix-Modbot")
                .send()
                .await
                .expect("Unable to login");

            info!("Doing the initial sync…");
            client
                .sync_once(SyncSettings::new())
                .await
                .expect("Unable to sync");

            info!(
                "Logged in as {}, got device_id {}",
                response.user_id, response.device_id
            );
            
            // Code snippet from hebbot https://github.com/haecker-felix/hebbot
            // Try to accept room invite, if any
            if let Some(invited_room) = client.get_invited_room(&room) {
                invited_room
                    .accept_invitation()
                    .await
                    .expect("Matrix-Modbot could not join the invited room");
            }

            // Need to do another sync to make sure bot knows the newly joined rooms
            client
                .sync_once(SyncSettings::new())
                .await
                .expect("Unable to sync");

            let joined_room = client
                .get_joined_room(&room)
                .expect("Cannot get joined room");

            let swear_list = create_swear_list(creds.swear_list_url.as_str())
                .expect("Could not download swear list");

            let bot = Bot {
                client,
                joined_room,
                swear_list
            };

            bot.send_message("Sucessfully started Matrix-Modbot").await;
            
            bot.client.add_event_handler_context(bot.clone());
            bot.client.add_event_handler(Self::on_room_message);

            info!("Started syncing");
            bot.client.sync(SyncSettings::new()).await.unwrap();
        }

        pub async fn on_room_message(event: OriginalSyncRoomMessageEvent, 
                                     room: Room, Ctx(bot): Ctx<Bot>) {
            if let Room::Joined(_joined) = &room {
                // Standard text message
                if let Some(text) = get_message_event_text(&event) {
                    let member = room.get_member(&event.sender).await.unwrap().unwrap();
                    let id = &event.event_id;

                }
                // Message edit
                if let Some((edited_msg_event_id, text)) = get_edited_message_event_text(&event) {}
            }
        }
            
        pub async fn send_message(&self, message: &str) {
            debug!("Send message ({:?}): {}", &self.joined_room, message);
            let content = RoomMessageEventContent::text_plain(message);
            let _ = &self.joined_room.send(content, None)
                .await
                .expect("Unable to send message");

        }

    }
}
